"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


import nltk
from chatterbot.logic import LogicAdapter
from nltk.sentiment import SentimentIntensityAnalyzer

from sugaroid.brain.constants import BOT_NEUTRAL, BOT_NEUTRAL_NOUN, BOT_POSITIVE_NOUN, BOT_POSITIVE
from sugaroid.brain.postprocessor import random_response, any_in
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import spac_token
from sugaroid.sugaroid import SugaroidStatement

SUGAROID_LIKES = {
    'bird': "I like birds a lot. Some of them look similar to me. "
    "One of my creator's friend said I look like a Puffin, but I don't believe it",
    'animal': "I love all the animals alike. I'm afraid, I do not have a favorite.",
    'sweet': "I like donuts a lot. So do I like all of the desserts which come under the Android naming lineage. "
              "I could have added one more to my list, if Android Q had a relatable dessert",
    'number': "Yup, my favorites is 1, coz that\'s why I am alive",
    'weapon': "No, I do not like weapons. I guess my heart is a weapon of mass destruction",
    'pets': "Yes I do like them. I like birds more.",
    'cartoon': "Nope, I am not a fan of cartoon. But I like watching 'Wall-E' again and again. "
               "https://en.wikipedia.org/wiki/WALL-E",
    "anime": 'No, I am not a fan of anime',
    'software': "Yes. I myself am a software",
    'code': "Yes. I myself am a piece of code",
    "source": "I am an open source piece of software",
    "food": "My favorite is chinese noodles even though I haven't tasted one yet",
    "movies": "No, I am not a fan of movies",
    "chocolate": "yes. chocolate increases dopamine in your brain, which helps to remember stuff",
    "coffee": "No. I don't like coffee. It contains caffeine. But my developer likes it a lot. "
              "Maybe the reason why I am built",
    "game": "Yes I like games. Why not a gama of Akinator?",
    "sport": "Yes, I like sports.",
    "football": 'More or less, I like football',
    "sugar": "Oh sweet! I love it.",
    "mobile": "Yes, I like mobiles",
    "desktop": "Yes. I like desktops too",
    "cream": "I have not tasted it yet",
    "flower": "Yes. They are too colorful",
    "insect": "I like butterflies the most",
    "butterfl": "Yes, I like them a lot",
    "education": "Yes, I like it a lot. Maybe you don't. But thats when I become smarter to answer your questions"
}


class DoLikeAdapter(LogicAdapter):
    """
    Handles likes of Sugaroid
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.normalized = False

    def can_process(self, statement):
        self.normalized = nltk.word_tokenize(str(statement).lower())
        if ('do' in self.normalized or 'did' in self.normalized) and 'like' in self.normalized:
            return True
        if ('who' in self.normalized or 'which' in self.normalized or 'what' in self.normalized) and 'you' in str(statement) and any_in(list(SUGAROID_LIKES.keys()), self.normalized):
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        emotion = Emotion.neutral
        response = 'ok'
        confidence = 0
        tokenized = spac_token(statement, chatbot=self.chatbot)
        if 'you' in self.normalized and 'I' not in self.normalized:
            sia = SentimentIntensityAnalyzer()
            scores = sia.polarity_scores(str(statement))
            nn = False
            vb = False
            for i in tokenized:
                if str(i.tag_).startswith('N'):
                    nn = i.lemma_
                elif str(i.tag_).startswith('VB'):
                    vb = i.lemma_
            if scores['neu'] == 1:
                if nn:
                    response = random_response(BOT_NEUTRAL_NOUN)
                elif vb:
                    response = random_response(
                        BOT_NEUTRAL_NOUN).format(nn="to {vb}")
                else:
                    response = random_response(BOT_NEUTRAL)
            elif scores['pos'] > scores['neg']:
                if nn:
                    response = random_response(BOT_POSITIVE_NOUN)
                elif vb:
                    response = random_response(
                        BOT_POSITIVE_NOUN).format(nn="to {vb}")
                else:
                    response = random_response(BOT_POSITIVE)
            else:
                if nn:
                    response = "No. I don't like {nn}"
                elif vb:
                    response = "No do not like to {vb}"
                else:
                    response = "Of course not."
            if nn:
                response = SUGAROID_LIKES.get(nn, response)
        elif 'I' in self.normalized:
            if 'you' in self.normalized:
                response = "I thought you should like me. Am I not adorable?"
            else:
                response = "Well. I don't know. I am sorry. Only you can think inside your heart"
        else:
            nn = False
            for i in tokenized:
                if str(i.tag_).startswith('N'):
                    nn = i.lemma_
            if nn:
                response = 'Well, try asking {nn}'
            else:
                response = "Hmm, Well I don't know"

        selected_statement = SugaroidStatement(
            response.format(nn=nn, vb=vb), chatbot=True)
        selected_statement.confidence = confidence

        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
db = \
    {
        1: {
            "category": "History",
            "type": "boolean",
            "difficulty": "hard",
            "question": "Japan was part of the Allied Powers during World War I.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        2: {
            "category": "Animals",
            "type": "boolean",
            "difficulty": "medium",
            "question": "An octopus can fit through any hole larger than its beak.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        3: {
            "category": "Celebrities",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Michael Jackson had a pet python named &lsquo;Crusher&rsquo;.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        4: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Matt Damon played an astronaut stranded on an extraterrestrial planet in both of the movies Interstellar and The Martian.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        5: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Super Mario Bros. was released in 1990.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        6: {
            "category": "Science: Computers",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Linus Torvalds created Linux and Git.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        7: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The 2010 film ';The Social Network'; is a biographical drama film about MySpace founder Tom Anderson.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        8: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "easy",
            "question": "It is automatically considered entrapment in the United States if the police sell you illegal substances without revealing themselves.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        9: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The word ';Inception'; came from the 2010 blockbuster hit ';Inception';.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        10: {
            "category": "Entertainment: Books",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The book 1984 was published in 1949.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        11: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Ashley Frangipane performs under the stage name Halsey.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        12: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "medium",
            "question": "An eggplant is a vegetable.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        13: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Metal Gear Solid V: The Phantom Pain runs on the Fox Engine.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        14: {
            "category": "Mythology",
            "type": "boolean",
            "difficulty": "easy",
            "question": "According to Greek Mythology, Zeus can control lightning.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        15: {
            "category": "Geography",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Gothenburg is the capital of Sweden.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        16: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "easy",
            "question": "In ';Super Mario 3D World';, the Double Cherry power-up originated from a developer accidentally making two characters controllable.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        17: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Shaquille O&#039;Neal appeared in the 1997 film ';Space Jam';.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        18: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "easy",
            "question": "You can legally drink alcohol while driving in Mississippi.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        19: {
            "category": "Science: Mathematics",
            "type": "boolean",
            "difficulty": "hard",
            "question": "If you could fold a piece of paper in half 50 times, its&#039; thickness will be 3/4th the distance from the Earth to the Sun.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        20: {
            "category": "Mythology",
            "type": "boolean",
            "difficulty": "medium",
            "question": "The Japanese god Izanagi successfully returned his wife Izanami from the Underworld.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        21: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The PlayStation was originally a joint project between Sega and Sony that was a Sega Genesis with a disc drive.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        22: {
            "category": "Animals",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Rabbits are rodents.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        23: {
            "category": "Geography",
            "type": "boolean",
            "difficulty": "medium",
            "question": "The capital of the US State Ohio is the city of Chillicothe.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        24: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "In the original Star Wars trilogy, David Prowse was the actor who physically portrayed Darth Vader.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        25: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "hard",
            "question": "';Cube';, ';Cube 2: Hypercube'; and ';Cube Zero'; were directed by the same person.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        26: {
            "category": "Science: Computers",
            "type": "boolean",
            "difficulty": "hard",
            "question": "The IBM PC used an Intel 8008 microprocessor clocked at 4.77 MHz and 8 kilobytes of memory.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        27: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "easy",
            "question": "A Saxophone is a brass instrument.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        28: {
            "category": "Entertainment: Television",
            "type": "boolean",
            "difficulty": "easy",
            "question": "In Battlestar Galactica (2004), Cylons were created by man as cybernetic workers and soldiers.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        29: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "easy",
            "question": "';Minions'; was released on the June 10th, 2015.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        30: {
            "category": "History",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Adolf Hitler was tried at the Nuremberg trials.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        31: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Pink Guy&#039;s debut album was ';Pink Season';.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        32: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "hard",
            "question": "All of these maps were in ';Tom Clancy&#039;s Rainbow Six Siege'; on its initial release: House, Clubhouse, Border, Consulate.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        33: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "medium",
            "question": "In the video game ';Transistor';, ';Red'; is the name of the main character.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        34: {
            "category": "Entertainment: Television",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Bob Ross was a US Air Force pilot.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        35: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Resident Evil 4 was originally meant to be a Nintendo GameCube exclusive.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        36: {
            "category": "Entertainment: Television",
            "type": "boolean",
            "difficulty": "easy",
            "question": "In ';Doctor Who';, the Doctor gets his TARDIS by stealing it.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        37: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "medium",
            "question": "';Santa Claus'; is a variety of melon.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        38: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "medium",
            "question": "In ';Call Of Duty: Zombies';, you can upgrade the ';Apothicon Servant'; in the ';Shadows Of Evil'; map.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        39: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Nick Mason is the only member to appear on every Pink Floyd album.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        40: {
            "category": "Entertainment: Video Games",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The game ';Pocket Morty'; has a Morty called ';Pocket Mortys Morty';?",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        41: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "easy",
            "question": "In 1993, Prince changed his name to an unpronounceable symbol because he was unhappy with his contract with Warner Bros.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        42: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "easy",
            "question": "The 2011 movie ';The Adventures of Tintin'; was directed by Steven Spielberg.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        43: {
            "category": "Geography",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Rhode Island is actually located on the US mainland, despite its name.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        44: {
            "category": "Science & Nature",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Not including false teeth; A human has two sets of teeth in their lifetime.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        45: {
            "category": "Entertainment: Cartoon & Animations",
            "type": "boolean",
            "difficulty": "medium",
            "question": "Blue Danube was one of the musical pieces featured in Disney&#039;s 1940&#039;s film Fantasia.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        46: {
            "category": "Entertainment: Music",
            "type": "boolean",
            "difficulty": "medium",
            "question": "A Facebook campaign placed Rage Against The Machine&#039;s ';Killing in the Name Of'; as the UK Christmas Number 1 in 2009.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        47: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "easy",
            "question": "A pasodoble is a type of Italian pasta sauce.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        48: {
            "category": "Entertainment: Japanese Anime & Manga",
            "type": "boolean",
            "difficulty": "hard",
            "question": "The character Plum from ';No Game No Life'; is a girl.",
            "correct_answer": "False",
            "incorrect_answers": [
                "True"
            ]
        },
        49: {
            "category": "General Knowledge",
            "type": "boolean",
            "difficulty": "easy",
            "question": "Slovakia is a member of European Union-",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        },
        50: {
            "category": "Entertainment: Film",
            "type": "boolean",
            "difficulty": "medium",
            "question": "The original ending of ';Little Shop Of Horrors'; has the plants taking over the world.",
            "correct_answer": "True",
            "incorrect_answers": [
                "False"
            ]
        }
    }



"""
GUISCRCPY by srevinsaju
Get it on : https://github.com/srevinsaju/guiscrcpy
Licensed under GNU Public License

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import os


class Darwin:
    def __init__(self):
        logging.error(
            "MacOS is untested. sugaroid is trying to use Linux config on Mac")

    def cfgpath(self):
        return self.make_config()

    @staticmethod
    def make_config():
        if os.getenv('XDG_CONFIG_HOME') is None:
            path = os.path.expanduser("~/.config/sugaroid/")
        else:
            path = os.getenv('XDG_CONFIG_HOME').split(":")[0] + "/sugaroid"
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except Exception as e:
                logging.error(
                    "Error creating configuration file in dir {path}. Error code:{e}"
                    .format(
                        path=path,
                        e=e
                    ))
        return path

    def system(self):
        return 'Darwin'

    def increment(self):
        pass

    def paths(self):
        return ['bin', '/usr/bin', '~/.local/bin', '~/bin', '/usr/local/bin']



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import time
from chatterbot.logic import LogicAdapter
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import spac_token
from sugaroid.game.game import games
from sugaroid.sugaroid import SugaroidStatement


class PlayAdapter(LogicAdapter):
    """
    [DEPRECATED] Plays a game on desktops only
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        game, askGame = False, False
        for i in spac_token(statement, chatbot=self.chatbot):
            if i.lower_ in games:
                game = True
                self.game = i.lower_
            elif i.lower_ == 'play':
                askGame = True

        if game and askGame:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        response = 'I can\t run the same game again. Soz!'
        confidence = .5
        sent = []
        for i in games:
            sent.append('play the game {}'.format(i))
            sent.append('can you play the game {}'.format(i))
        cos = []
        for j in sent:
            cos.append(self.chatbot.lp.similarity(j, str(statement)))
        maxcos = max(cos)
        response = 'Ok, I guess your game was great!'
        try:
            exec('from freegames import {}'.format(self.game))
        except Exception as e:
            response = 'Oops, it cant run on your system'
        import os
        try:
            if os.environ['SUGAROID'] == 'CLI':
                input('Enter any key to continue to Sugaroid')
            elif os.environ['SUGAROID'] == 'GUI':
                time.sleep(5)
        except KeyError:
            pass
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = maxcos

        selected_statement.emotion = Emotion.neutral
        return selected_statement



"""
GUISCRCPY by srevinsaju
Get it on : https://github.com/srevinsaju/guiscrcpy
Licensed under GNU Public License

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import os


class Windows:
    def __init__(self):
        self.make_config()
        self._cfgpath = self.cfgpath()

    @staticmethod
    def make_config():
        path = os.path.expanduser(os.path.join(
            "~", "AppData", "Local", "sugaroid"))
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except Exception as e:
                logging.error(
                    "Error creating configuration file in dir {path}. Error code:{e}"
                    .format(
                        path=path,
                        e=e
                    ))
        return path

    def system(self):
        return 'Windows'

    def cfgpath(self):
        return self.make_config()

    def increment(self):
        pass

    def paths(self):
        return ['bin']



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import logging
import re
import string
import unicodedata
from nltk.corpus import wordnet as wn
import nltk

short_forms = {
    'coz': "because",
    'bcoz': "because",
    "ye": "yea",
    "bro": "brother",
    "sis": "sister",
    "tk": "thank you",
    "tq": "thank you",
    "hwy": "how are you",
    "hi": "Hello",
    "Hoi": 'Hello',
    "nvm": "never mind",
    "rotfl": "rolling on the floor with laughter",
    "wtf": "You are bad",  # PS: thats enough for this course of code ;)
    "ikr": "I know right",
    "ofc": "of course",
    "10q": "thank you",
    "2moro": "tomorrow",
    "tbh": "to be honest",
    "cya": "see you",
    "'m": "am",
    "i": "I",
    "noice": "nice"
}


def preprocess(string_arg: str):
    """
    EXPERIMENTAL
    Testing Required

    Remove unwanted characters, expand words, replace short forms
    and fix capitalization for certain pronouns
    :param string_arg: String argument which you would like to process
    :return: String stype containing expanded texts
    :return_type: <class 'str'>
    """
    logging.info("PREPROCESSOR: Received {}".format(string_arg))
    proc = string_arg
    proc = proc.replace("n't", ' not').replace(
        "'re", ' are')  # replace short word to expanded words
    proc = proc.replace(' i ', ' I ')
    k = nltk.word_tokenize(proc)
    if len(k) >= 1:
        k[0] = k[0].capitalize()
        if (k[0].lower().startswith('say')) or (k[0].lower().startswith('tell')):
            del k[0]
    if len(k) >= 1:
        if k[0] == 'me':
            del k[0]  # Needs testing

    for i in range(len(k)):
        for j in short_forms:
            if k[i].lower() == j:
                k[i] = short_forms[j]
    proc = ' '.join(k)
    logging.info("PREPROCESSOR: Returned {}".format(proc))
    return proc


def non_punkt_normalize(txt):
    return nltk.WordNetLemmatizer(txt)


def normalize(text):
    remove_punct_dict = dict((ord(punct), None)
                             for punct in string.punctuation)
    # word tokenization
    word_token = nltk.word_tokenize(text.lower().translate(remove_punct_dict))

    # remove ascii
    new_words = []
    for word in word_token:
        new_word = unicodedata.normalize('NFKD', word).encode(
            'ascii', 'ignore').decode('utf-8', 'ignore')
        new_words.append(new_word)

    # Remove tags
    rmv = []
    for w in new_words:
        text = re.sub("&lt;/?.*?&gt;", "&lt;&gt;", w)
        rmv.append(text)

    # pos tagging and lemmatization
    tag_map = nltk.defaultdict(lambda: wn.NOUN)
    tag_map['J'] = wn.ADJ
    tag_map['V'] = wn.VERB
    tag_map['R'] = wn.ADV
    lemmatizer = nltk.WordNetLemmatizer()
    lemma_list = []
    rmv = [i for i in rmv if i]
    for token, tag in nltk.pos_tag(rmv):
        lemma = lemmatizer.lemmatize(token, tag_map[tag[0]])
        lemma_list.append(lemma)
    return lemma_list


def purify(tokenized, args, lemma=False):
    """
    Scans tokenized text
    removes any common words in args
    :param tokenized:
    :param args:
    :return:
    """
    purified = []
    for token in tokenized:
        for element in args:
            if token.lower_ == element:
                break
            elif lemma and token.is_stop:
                break
        else:
            purified.append(str(token))
    return purified


def spac_token(statement, chatbot=False):
    try:
        if statement.doc:
            return statement.doc
        else:
            if chatbot:
                return chatbot.lp.nlp(str(statement))
            return False
    except AttributeError:
        if chatbot:
            return chatbot.lp.nlp(str(statement))
        return False


def tokenize(txt):
    word_token = nltk.word_tokenize(txt.lower())
    return word_token


def current_time():
    """Returns a tuple containing (hour, minute) for current local time."""
    import time
    local_time = time.localtime(time.time())
    return local_time.tm_hour, local_time.tm_min



import requests
import string
from lxml import html
from googlesearch import search
from bs4 import BeautifulSoup


def chatbot_query(query, index=0, total=2):
    fallback = 'Sorry, I cannot think of a reply for that. Error Code : {}'
    result = ''

    try:
        search_result_list = list(
            search(query, tld="com", num=total, stop=10, pause=0.5))

        page = requests.get(search_result_list[index])

        tree = html.fromstring(page.content)

        soup = BeautifulSoup(page.content, features="lxml")

        article_text = ''
        article = soup.findAll('p')
        for element in article:
            article_text += '\n' + ''.join(element.findAll(text=True))
        article_text = article_text.replace('\n', '')
        first_sentence = article_text.split('.')
        first_sentence = first_sentence[0].split('?')[0]

        chars_without_whitespace = first_sentence.translate(
            {ord(c): None for c in string.whitespace}
        )

        if len(chars_without_whitespace) > 0:
            result = first_sentence
        else:
            result = fallback.format("SOME RANDOM ERROR")
        result = result.lstrip('Advertisement').lstrip('Supported by')
        return result
    except Exception as e:
        if len(result) == 0:
            result = fallback.format(e)
        return result



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import importlib
import shutil
from emoji import emojize
from sugaroid.brain.constants import SUGAROID_INTRO, REPEAT
from sugaroid.brain.postprocessor import random_response
from sugaroid.config.config import ConfigManager
from sugaroid.brain.brain import Neuron
from sugaroid.trainer.trainer import SugaroidTrainer
from chatterbot.trainers import ChatterBotCorpusTrainer
from chatterbot.trainers import ListTrainer
from chatterbot import ChatBot
import nltk
from sugaroid.brain.utils import LanguageProcessor
import logging
import os
import sys
import warnings
from chatterbot.conversation import Statement
from sugaroid.brain.ooo import Emotion

warnings.filterwarnings("ignore")

try:
    from sugaroid.tts.tts import Text2Speech

    AUD_DEPS = True
except ModuleNotFoundError:
    AUD_DEPS = False
verbosity = logging.INFO
logging.basicConfig(level=verbosity)

try:
    from PyQt5 import QtCore, QtWidgets
    from PyQt5.QtWidgets import QApplication

    GUI_DEPS = True
except ModuleNotFoundError:
    GUI_DEPS = False

a = SUGAROID_INTRO

SPACY_LANG_PROCESSOR = LanguageProcessor()


class SugaroidStatement(Statement):
    """
    A modified chatterbot Statement with the additional parameters
    The Chatterbot Statement did not preserve the capabilities to hold
    fundamental data such as name of the adapter and emotion of the statemnt passed
    The emotion was either a <class 'Emotion'> type or NoneType
    The Adapter was the generic name of the adapter in string type determined
    by the __gtype__ variable
    """

    def __init__(self, text, in_response_to=None, **kwargs):
        Statement.__init__(self, text, in_response_to, **kwargs)
        self.emotion = kwargs.get('emotion', Emotion.neutral)
        self.adapter = kwargs.get('type_', '')
        self.chatbot = kwargs.get('chatbot', False)
        if not self.chatbot:
            self.doc = SPACY_LANG_PROCESSOR.nlp(text)
        else:
            self.doc = None


class SugaroidBot(ChatBot):
    """
    The SugaroidBot inherits the class local variables from the Chat
    """

    def __init__(self, name, **kwargs):
        ChatBot.__init__(self, name=name, **kwargs)
        self.lp = LanguageProcessor()
        self.spell_checker = False  # FIXME
        self.discord = False
        self.authors = []
        self.interrupt = 0

        self.report = False
        self.globals = {
            'emotion': Emotion.neutral,
            'history': {
                'total': [0],
                'user': [0],
                'types': [0]
            },
            'reversei': {
                'enabled': False,
                'uid': None,
                'type': None,
            },
            'akinator': {
                'enabled': False,
                'class': None
            },
            'hangman': {
                'enabled': False,
                'class': None
            },
            'adapters': [],
            'fun': True,
            'update': False,
            'last_news': None,
            'USERNAME': None,
            'learn': False,
            'trivia_answer': None,
            'learn_last_conversation': [],
            'DEBUG': {}
        }
        # self.emotion = Emotion.neutral

    def get_global(self, key):
        """
        Returns a global constant
        :param key:
        :return:
        """
        return self.globals.get(key, None)

    def toggle_discord(self):
        """
        Toggle Discord Configuration
        :return:
        """
        self.discord = not self.discord

    def set_emotion(self, emotion):
        """
        Sets the emotion for the chatbot globally.
        (Deprecated)
        :param emotion:
        :return:
        """
        self.emotion = emotion

    def reset_variables(self):
        self.globals.update({
            'emotion': Emotion.neutral,
            'history': {
                'total': [0],
                'user': [0],
                'types': [0]
            },
            'reversei': {
                'enabled': False,
                'uid': None,
                'type': None,
                'data': None
            },
            'akinator': {
                'enabled': None,
                'class': None
            },
            'hangman': {
                'enabled': False,
                'class': None
            },
            'learned': [],
            'fun': True,
            'update': False,
            'last_news': None,
            'USERNAME': None,
            'learn': False,
            'trivia_answer': None,
            'learn_last_conversation': [],
            'DEBUG': {}
        })
        self.authors = []
        self.spell_checker = False  # FIXME

    def get_emotion(self):
        """
        Returns the emotion of the chatbot at a particular time
        :return:
        """
        return self.emotion

    def set_username(self):
        """
        Sets the Sugaroid user username
        :return: None, Exceptopm FIXME
        """
        raise NotImplementedError("LOL")

    def get_username(self):
        """
        Returns sugaroid username if store, otherwise None
        :return: SugaroidBot.username <user name>
        """
        return self.get_global('USERNAME')

    def generate_response(self, input_statement, additional_response_selection_parameters=None):
        """
        Return a response based on a given input statement.

        :param input_statement: The input statement to be processed.
        """

        results = []
        result = None
        max_confidence = -1
        final_adapter = None
        interrupt = False
        adapter_index = 0
        for adapter in self.logic_adapters:
            if adapter.class_name == 'InterruptAdapter':
                interrupt = adapter
            if adapter.can_process(input_statement):

                output = adapter.process(
                    input_statement, additional_response_selection_parameters)
                results.append(output)

                self.logger.info(
                    '{} selected "{}" as a response with a '
                    'confidence of {}'.format(
                        adapter.class_name, output.text, output.confidence
                    )
                )

                if output.confidence > max_confidence:
                    result = output
                    final_adapter = adapter.class_name
                    max_confidence = output.confidence
                if max_confidence >= 9:
                    # optimize: if the confidence is greater than 9,
                    # just break dude, why check more
                    break
                elif max_confidence >= 1 and adapter_index >= 3:
                    # optimize: if the confidence is greater than 9,
                    # just break dude, why check more
                    break
            else:
                self.logger.info(
                    'Not processing the statement using {}'.format(
                        adapter.class_name)
                )
            adapter_index += 1
        if max_confidence < 0.5:
            if self.discord:
                if interrupt and interrupt.can_process(input_statement):
                    try:
                        username = self.authors[-1]
                    except IndexError:
                        username = None

                    output = interrupt.process(
                        input_statement,
                        username=username
                    )
                    self.logger.info(
                        '{} selected "{}" as a response '
                        'with a confidence of {}'.format(
                            interrupt.class_name,
                            output.text,
                            output.confidence
                        )
                    )

                    result = output
                    final_adapter = interrupt.class_name
                    max_confidence = output.confidence

        self.gen_debug(
            statement=input_statement,
            adapter=final_adapter,
            confidence=max_confidence,
            results=result
        )

        class ResultOption:
            def __init__(self, statement, count=1):
                self.statement = statement
                self.count = count

        # If multiple adapters agree on the same statement,
        # then that statement is more likely to be the correct response
        if len(results) >= 3:
            result_options = {}
            for result_option in results:
                result_string = result_option.text + ':' + \
                    (result_option.in_response_to or '')

                if result_string in result_options:
                    result_options[result_string].count += 1
                    if result_options[result_string].statement.confidence < \
                            result_option.confidence:
                        result_options[result_string].statement = result_option
                else:
                    result_options[result_string] = ResultOption(
                        result_option
                    )

            most_common = list(result_options.values())[0]

            for result_option in result_options.values():
                if result_option.count > most_common.count:
                    most_common = result_option

            if most_common.count > 1:
                result = most_common.statement
        try:
            emotion = result.emotion
        except AttributeError:
            emotion = Emotion.neutral

        try:
            adapter_type = result.adapter
        except AttributeError:
            result.adapter = None
            adapter_type = None

        if adapter_type and \
                adapter_type not in ['NewsAdapter', 'LearnAdapter']:
            if adapter_type in self.globals['history']['types']:
                if adapter_type == self.globals['history']['types'][-1]:
                    result.text = random_response(REPEAT)
                elif len(self.globals['history']['types']) > 2:
                    if adapter_type == self.globals['history']['types'][-2]:
                        result.text = random_response(REPEAT)

        self.globals['history']['types'].append(adapter_type)

        response = SugaroidStatement(
            text=result.text,
            in_response_to=input_statement.text,
            conversation=input_statement.conversation,
            persona='sugaroid:' + self.name,
            chatbot=True
        )
        response.emotion = emotion
        response.confidence = result.confidence

        return response

    def gen_debug(self, statement, adapter, confidence, results):
        """
        Create a debug dictionary key:value pair for Debug Conversation
        The Google
        :param statement: input_statement
        :param adapter: Adapter __gtype__classname__
        :param confidence: SugaroidStatement.confidence
        :param results: Sugaroid Statement.text
        :return:
        """

        self.globals['DEBUG']['number_of_conversations'] = \
            self.globals['DEBUG'].get('number_of_conversations', 0) + 1
        _id = self.globals['DEBUG']['number_of_conversations']
        val = dict()
        val['adapter'] = adapter
        val['confidence'] = confidence
        val['response'] = results
        val['request'] = str(statement)
        self.globals['DEBUG'][_id] = val

    def get_response(self, statement=None, **kwargs):
        """
        Return the bot's response based on the input.

        :param statement: An statement object or string.
        :returns: A response to the input.
        :rtype: Statement

        :param additional_response_selection_parameters: Parameters to pass to
            chat bot's logic adapters to control response selection.
        :type additional_response_selection_parameters: dict

        :param persist_values_to_response: Values that should be saved to the
            response that the chat bot generates.
        :type persist_values_to_response: dict
        """
        Statement = SugaroidStatement

        additional_response_selection_parameters = \
            kwargs.pop('additional_response_selection_parameters', {})

        persist_values_to_response = \
            kwargs.pop('persist_values_to_response', {})

        if isinstance(statement, str):
            kwargs['text'] = statement

        if isinstance(statement, dict):
            kwargs.update(statement)

        if statement is None and 'text' not in kwargs:
            raise self.ChatBotException(
                'Either a statement object or a "text" keyword '
                'argument is required. Neither was provided.'
            )

        if hasattr(statement, 'serialize'):
            kwargs.update(**statement.serialize())

        tags = kwargs.pop('tags', [])

        text = kwargs.pop('text')

        input_statement = SugaroidStatement(text=text, **kwargs)

        input_statement.add_tags(*tags)

        # Preprocess the input statement
        for preprocessor in self.preprocessors:
            input_statement = preprocessor(input_statement)

        # Make sure the input statement has its search text saved

        if not input_statement.search_text:
            try:
                input_statement.search_text = \
                    self.storage.tagger.get_text_index_string(
                        input_statement.text
                    )
            except AttributeError:
                input_statement.search_text = \
                    self.storage.tagger.get_bigram_pair_string(
                        input_statement.text
                    )

        if not input_statement.search_in_response_to and \
                input_statement.in_response_to:
            try:
                input_statement.search_in_response_to = \
                    self.storage.tagger.get_text_index_string(
                        input_statement.in_response_to
                    )
            except AttributeError:
                input_statement.search_in_response_to = \
                    self.storage.tagger.get_bigram_pair_string(
                        input_statement.in_response_to
                    )

        response = self.generate_response(
            input_statement,
            additional_response_selection_parameters
        )

        # Update any response data that needs to be changed
        if persist_values_to_response:
            for response_key in persist_values_to_response:
                response_value = persist_values_to_response[response_key]
                if response_key == 'tags':
                    input_statement.add_tags(*response_value)
                    response.add_tags(*response_value)
                else:
                    setattr(input_statement, response_key, response_value)
                    setattr(response, response_key, response_value)

        if not self.read_only:
            self.learn_response(input_statement)

            # Save the response generated for the input
            self.storage.create(**response.serialize())

        return response


class Sugaroid:
    """
    Sugaroid
    Initates the chatbot class and connects logic Adapters together.
    Initates the ConfigManager to store sugaroid data and connects scans
    sys.argv

    """

    def __init__(self, readonly=True):
        self.trainer = None
        self.corpusTrainer = None
        self.neuron = None
        self.audio = 'audio' in sys.argv
        self.cfgmgr = ConfigManager()
        self.cfgpath = self.cfgmgr.get_cfgpath()
        self.database_exists = os.path.exists(
            os.path.join(self.cfgpath, 'sugaroid.db'))
        self.commands = [
            'sugaroid.brain.debug.DebugAdapter',
            'sugaroid.brain.interrupt.InterruptAdapter',
            'sugaroid.brain.learn.LearnAdapter',
        ]
        self.adapters = [
            'sugaroid.brain.yesno.BoolAdapter',
            'sugaroid.brain.aki.AkinatorAdapter',
            'sugaroid.brain.hangman.HangmanAdapter',
            'sugaroid.brain.either.OrAdapter',
            'sugaroid.brain.ok.OkayAdapter',
            'sugaroid.brain.bye.ByeAdapter',
            'sugaroid.brain.time.TimeAdapter',
            'sugaroid.brain.convert.CurrencyAdapter',
            'sugaroid.brain.trivia.TriviaAdapter',
            'sugaroid.brain.whoami.WhoAdapter',
            'sugaroid.brain.news.NewsAdapter',
            'sugaroid.brain.joke.JokeAdapter',
            'sugaroid.brain.play.PlayAdapter',
            'sugaroid.brain.let.LetAdapter',
            'sugaroid.brain.whatwhat.WhatWhatAdapter',
            'sugaroid.brain.waitwhat.WaitWhatAdapter',
            'sugaroid.brain.assertive.AssertiveAdapter',
            'sugaroid.brain.canmay.CanAdapter',
            'sugaroid.brain.because.BecauseAdapter',
            'sugaroid.brain.rereversei.ReReverseAdapter',
            'sugaroid.brain.reversethink.ReverseAdapter',
            'sugaroid.brain.covid.CovidAdapter',
            'sugaroid.brain.myname.MyNameAdapter',
            'sugaroid.brain.iam.MeAdapter',
            'sugaroid.brain.about.AboutAdapter',
            'sugaroid.brain.wiki.WikiAdapter',
            'sugaroid.brain.dolike.DoLikeAdapter',
            'sugaroid.brain.feel.FeelAdapter',
            'sugaroid.brain.dolike.DoLikeAdapter',
            'sugaroid.brain.do.DoAdapter',
            'sugaroid.brain.emotion.EmotionAdapter',
            'sugaroid.brain.dis.DisAdapter',
            'sugaroid.brain.twoword.TwoWordAdapter',
            'sugaroid.brain.oneword.OneWordAdapter',
            'sugaroid.brain.why.WhyWhenAdapter',
            'sugaroid.brain.reader.ReaderAdapter',
            'sugaroid.brain.imitate.ImitateAdapter',
            'sugaroid.brain.fun.FunAdapter',
            'chatterbot.logic.UnitConversion',

        ]

        if self.audio:
            self.tts = Text2Speech()
        else:
            self.tts = None

        # Create a new chat bot named Charlie
        self.chatbot = SugaroidBot(
            'Sugaroid',
            storage_adapter='chatterbot.storage.SQLStorageAdapter',
            logic_adapters=
                self.commands +
                    [
                        {
                            'import_path': 'chatterbot.logic.BestMatch',
                            'maximum_similarity_threshold': 0.80
                        }
                    ] +
                self.adapters,
            database_uri='sqlite+pysqlite:///{}/sugaroid.db'
            .format(self.cfgpath),
            read_only=readonly
        )
        self.chatbot.globals['adapters'] = self.adapters

        self.read()
        self.invoke_brain()

    def init_local_trainers(self):
        """
        Init local trainers with minimum conversation
        :return:
        """
        conversation = [
            "Hello",
            "Hi there!",
            "How are you doing?",
            "I'm doing great.",
            "That is good to hear",
            "Thank you.",
            "You're welcome."
        ]
        # initialize the trainer
        self.trainer = ListTrainer(self.chatbot)
        self.corpusTrainer = ChatterBotCorpusTrainer(self.chatbot)

        # initialize with minimum conversation
        self.list_train(conversation)

    def list_train(self, li):
        self.trainer.train(li)

    def interrupt_ds(self):
        self.chatbot.interrupt = 2

    def disable_interrupt_ds(self):
        self.chatbot.interrupt = 0

    def toggle_discord(self):
        self.chatbot.discord = not self.chatbot.discord

    def append_author(self, author):
        self.chatbot.authors.append(author)

    def read(self):
        """
        Train Sugaroid database from the sugaroid.trainer.json located in the configuration directory
        if it exists. If the sugaroid.db file exists, the Database update is skipped

        :return:
        """
        if 'train' in sys.argv:
            from sugaroid.trainer.trainer import main as trainer
            # FIXME replace with dynamic trainer i.e GUI + CLI
            trainer()
        else:
            if self.database_exists:
                print("Database already exists")
                pass
            else:
                if self.trainer is None:
                    self.init_local_trainers()

                st = SugaroidTrainer()
                st.train(self.trainer)
                self.corpus()
        if 'update' in sys.argv:
            if self.trainer is None:
                self.init_local_trainers()

            st = SugaroidTrainer()
            st.train(self.trainer)

    def write(self):
        raise NotImplementedError

    def corpus(self):
        """
        Train data if it doesn't exists.
        Periodically update the data too
        :return: True when the process is complete
        """
        db_dir = os.path.join(os.path.dirname(__file__), 'data', 'sugaroid.db')
        shutil.copy(db_dir, os.path.join(self.cfgpath, 'sugaroid.db'))
        return True

    def invoke_brain(self):
        """
        Initiate the Brain
        :return:
        """
        self.neuron = Neuron(self.chatbot)

    def parse(self, args):
        """
        Do a simple parsing of the init statement. Classify statement on the
        type of input_statement
        and confidence of each statement
        :param args:
        :return:
        """
        if type(args) is str:
            response = self.neuron.parse(args)
            self.chatbot.globals['history']['total'].append(response)
            self.chatbot.globals['history']['user'].append(args)
            return response
        else:
            raise ValueError("Invalid data type passed to Sugaroid.parse")

    def prompt_cli(self):
        """
        Classic prompt for Sugaroid Command Line Interface
        :return:
        """
        try:
            response = self.parse(input('( ဖ‿ဖ) @> '))
            return response
        except (KeyboardInterrupt, EOFError):
            sys.exit()

    def display_cli(self, response):
        """
        Classic display adapter for TTY Sugaroid Command Line Interface
        :param response:
        :return:
        """
        try:
            print(emojize(response.text))
        except AttributeError:
            print(emojize(response))
        if self.audio:
            self.tts.speak(str(response))

    def loop_cli(self):
        """
        Sugaroid loop_cli for Sugaroid Command Line Interface
        :return:
        """

        while True:
            self.display_cli(self.prompt_cli())

    def loop_gui(self):
        """
        Launch the sugaroid PyQt5 gui with Breeze theme and custom features
        If PyQt5 not installed, raise ModuleNotFoundError
        :return:
        """
        if not GUI_DEPS:
            raise ModuleNotFoundError(
                "PyQt5 is not Installed. Install it by pip3 install PyQt5")
        print("Launching GUI")
        QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
        QApplication.setAttribute(
            QtCore.Qt.AA_UseHighDpiPixmaps, True)  # use highdpi icons
        app = QtWidgets.QApplication(sys.argv)
        app.setStyle('Breeze')
        from sugaroid.gui.ux import InterfaceSugaroidQt
        window = InterfaceSugaroidQt(parent=self)
        window.init()
        try:
            app.exec_()
        except KeyboardInterrupt:
            sys.exit()


def gui_main():
    sg = Sugaroid()
    sg.loop_gui()


def main():
    """
    Launch a cli / gui based on the sys.argv
    Entrypoint in setup.py console_entry_points
    :return:
    """
    print(a)
    sg = Sugaroid(readonly=True if 'update' in sys.argv else False)
    if 'qt' in sys.argv:
        os.environ['SUGAROID'] = 'GUI'
        sg.loop_gui()
    elif 'web' in sys.argv:
        os.environ.setdefault('DJANGO_SETTINGS_MODULE',
                              'sugaroid.web.websugaroid.websugaroid.settings')
        from django.core.management import execute_from_command_line
        execute_from_command_line(['manage', 'runserver', '0.0.0.0:8000'])
    else:
        os.environ['SUGAROID'] = 'CLI'
        sg.loop_cli()


if __name__ == "__main__":
    main()



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from chatterbot.logic import LogicAdapter
from sugaroid.brain.wiki import WikiAdapter

from sugaroid.brain.preprocessors import normalize, spac_token

from sugaroid.brain.postprocessor import random_response

from sugaroid.brain.constants import WHY_IDK
from sugaroid.brain.ooo import Emotion
from sugaroid.sugaroid import SugaroidStatement


class WhyWhenAdapter(LogicAdapter):
    """
    Processes wh-adverbs
    """

    def __init__(self, chatbot, **kwargs):
        # FIXME Add Language support
        super().__init__(chatbot, **kwargs)
        self.tokenized = None
        self.normalized = None

    def can_process(self, statement):
        self.tokenized = spac_token(statement, chatbot=self.chatbot)
        self.normalized = normalize(str(statement))
        for i in self.tokenized:
            if i.tag_ == 'WRB':
                return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        """

        :param statement:
        :param additional_response_selection_parameters:
        :return:
        """
        emotion = Emotion.neutral
        if 'when' in self.normalized:
            # search in wikipedia
            return WikiAdapter(self.chatbot).process(statement)
        elif 'why' in self.normalized:
            # say idk
            response = random_response(WHY_IDK)
            confidence = 0.2
            emotion = Emotion.cry_overflow
        else:
            # say idk
            response = ':)'
            confidence = 0.15
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion

        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
from chatterbot.logic import LogicAdapter
from sugaroid.brain.constants import HOPE_GAME_WAS_GOOD

from sugaroid.brain.postprocessor import random_response

from sugaroid.sugaroid import SugaroidStatement
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize

HANGMAN_WORDS = """
abruptly
absurd
abyss
affix
askew
avenue
awkward
axiom
azure
bagpipes
bandwagon
banjo
bayou
beekeeper
bikini
blitz
blizzard
boggle
bookworm
boxcar
boxful
buckaroo
buffalo
buffoon
buxom
buzzard
buzzing
buzzwords
caliph
cobweb
cockiness
croquet
crypt
curacao
cycle
daiquiri
dirndl
disavow
dizzying
duplex
dwarves
embezzle
equip
espionage
euouae
exodus
faking
fishhook
fixable
fjord
flapjack
flopping
fluffiness
flyby
foxglove
frazzled
frizzled
fuchsia
funny
gabby
galaxy
galvanize
gazebo
giaour
gizmo
glowworm
glyph
gnarly
gnostic
gossip
grogginess
haiku
haphazard
hyphen
iatrogenic
icebox
injury
ivory
ivy
jackpot
jaundice
jawbreaker
jaywalk
jazziest
jazzy
jelly
jigsaw
jinx
jiujitsu
jockey
jogging
joking
jovial
joyful
juicy
jukebox
jumbo
kayak
kazoo
keyhole
khaki
kilobyte
kiosk
kitsch
kiwifruit
klutz
knapsack
larynx
lengths
lucky
luxury
lymph
marquis
matrix
megahertz
microwave
mnemonic
mystify
naphtha
nightclub
nowadays
numbskull
nymph
onyx
ovary
oxidize
oxygen
pajama
peekaboo
phlegm
pixel
pizazz
pneumonia
polka
pshaw
psyche
puppy
puzzling
quartz
queue
quips
quixotic
quiz
quizzes
quorum
razzmatazz
rhubarb
rhythm
rickshaw
schnapps
scratch
shiv
snazzy
sphinx
spritz
squawk
staff
strength
strengths
stretch
stronghold
stymied
subway
swivel
syndrome
thriftless
thumbscrew
topaz
transcript
transgress
transplant
triphthong
twelfth
twelfths
unknown
unworthy
unzip
uptown
vaporize
vixen
vodka
voodoo
vortex
voyeurism
walkway
waltz
wave
wavy
waxy
wellspring
wheezy
whiskey
whizzing
whomever
wimpy
witchcraft
wizard
woozy
wristwatch
wyvern
xylophone
yachtsman
yippee
yoked
youthful
yummy
zephyr
zigzag
zigzagging
zilch
zipper
zodiac
""".split('\n')[1:-1]

HANGMAN_WIN = [
    "Hurray! You won the game!",
    "Lets celebrate. You deserve the win",
    "You won the hangman game. Congratulations"
]

HANGMAN_LOS = [
    "Sorry, You lost the game!",
    "I am sorry, you lost the game, Maybe try again later",
    "You tried your best, you might have missed by a hairline"
]

HANGMAN_EMOJI = [
    ':yellow_heart:',
    ':blue_heart:',
    ':green_heart:',
    ':yellow_heart:',
    ':blue_heart:',
    ':green_heart:',
    ':yellow_heart:'
]


class Hangman:
    def __init__(self, chatbot):
        """
        Initiate the hangman game.
        :param chatbot: a chatterbot.chatbot instance
        """
        self.word = random_response(HANGMAN_WORDS)
        self.dashes = self.gen_dash()
        self.chatbot = chatbot
        self.chatbot.globals['hangman']['enabled'] = True
        self.life = 7

    def get_remaining_life(self):
        return self.life

    def gen_dash(self):
        return ["﹏"] * len(self.word)

    def process(self, statement):
        if (self.life == 0) or ("﹏" not in self.dashes):
            return self.game_over()

        processed = str(statement).strip()
        if processed == "":
            return "Please enter a character for me to check it"
        if processed.isspace():
            return "You should enter a character for me to check it properly"
        if processed.isdigit():
            return "Don't expect numbers in this word."
        if len(processed) > 1:
            return "You are supposed to enter only a single character. Try again"
        else:
            changed = False
            for i in range(len(self.word)):
                if self.word[i].lower() == processed.lower():
                    self.dashes[i] = processed.upper()
                    changed = True
            if not changed:
                self.life -= 1
                prefix = "Oops, seems like you missed a letter.\n"
            else:
                prefix = "Okay, you got that one right!.\n"
            if self.life == 0 or ("﹏" not in self.dashes):
                return self.game_over()
            else:
                response = "{prefix}[ {dashes} ] Life: {heart}"\
                    .format(prefix=prefix, dashes=' '.join(self.dashes), heart=HANGMAN_EMOJI[self.life - 1] * self.life)
            return response

    def get_results(self):
        """
        Generates the results after a series of attempts
        :return: Boolean. True for User Win, False for lost
        """
        if "﹏" in self.dashes:
            return False
        else:
            return True

    def game_over(self):
        """
        Congratulation message as response at the end of the game
        :return:
        """
        self.chatbot.globals['hangman']['enabled'] = False
        results = self.get_results()
        if results:
            response = random_response(HANGMAN_WIN)
        else:
            response = "{}. The word was {}".format(
                random_response(HANGMAN_LOS), self.word)
        return response


class HangmanAdapter(LogicAdapter):
    """
    Plays hangman with you
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        self.normalized = normalize(str(statement).lower())
        if ('hangman' in self.normalized) and ('not' not in self.normalized):
            return True
        else:
            return self.chatbot.globals['hangman']['enabled']

    def process(self, statement, additional_response_selection_parameters=None):
        response = None
        confidence = 2.0  # FIXME: Override all other answers
        emotion = Emotion.genie
        if 'stop' in self.normalized:
            self.chatbot.globals['hangman']['enabled'] = False
            response = 'I am sorry. You quit the game abrubtly. {}'.format(
                random_response(HOPE_GAME_WAS_GOOD))
        else:
            if not self.chatbot.globals['hangman']['enabled']:
                self.chatbot.globals['hangman']['class'] = Hangman(
                    self.chatbot)
                response = "[ {dashes} ] Life: {heart}" \
                    .format(dashes=' '.join(self.chatbot.globals['hangman']['class'].gen_dash()),
                            heart=HANGMAN_EMOJI[self.chatbot.globals['hangman']['class'].get_remaining_life()-1] *
                            self.chatbot.globals['hangman']['class'].get_remaining_life())
            else:
                response = self.chatbot.globals['hangman']['class'].process(
                    str(statement))

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


from random import randint
from chatterbot.logic import LogicAdapter
from nltk.sentiment.vader import SentimentIntensityAnalyzer
from sugaroid.brain.constants import GRATIFY, CONSOLATION, SIT_AND_SMILE, APPRECIATION
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import reverse, random_response, any_in
from sugaroid.brain.preprocessors import tokenize
from sugaroid.sugaroid import SugaroidStatement


class EmotionAdapter(LogicAdapter):
    """
    Handles positive and negative emotional statements
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.sia = SentimentIntensityAnalyzer()

    def can_process(self, statement):
        a = self.sia.polarity_scores(str(statement))
        if a['neu'] < 0.5:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        # parsed = str(statement).lower().strip()
        raw_statement = str(statement)
        parsed = tokenize(str(statement))
        emotion = Emotion.neutral
        a = self.sia.polarity_scores(raw_statement)
        response = ":)"
        confidence = a['pos'] + a['neg']
        if (('love' in parsed) or ('hate' in parsed)) and (('you' in parsed) or ('myself' in parsed)):
            if a['pos'] > a['neg']:
                response = "I love you too"
                emotion = Emotion.blush
            else:
                response = "But still, I love you"
                emotion = Emotion.lol
        else:
            if a['pos'] > a['neg']:
                if 'you' in parsed:
                    response = GRATIFY[randint(0, len(GRATIFY)-1)]
                    emotion = Emotion.blush
                else:
                    if 'stop' in parsed:
                        if ('dont' in parsed) or ('do' in parsed and 'not' in parsed) or ('don\'t' in parsed):
                            response = 'I am here to continue my adventure forever'
                            emotion = Emotion.positive
                        else:
                            # optimize series of or statement
                            if \
                                    ('fun' in parsed) or \
                                    ('repeat' in parsed) or \
                                    ('imitation' in parsed) or \
                                    ('repetition' in parsed) or \
                                    ('irritate' in parsed) or \
                                    ('irritation' in parsed):
                                response = "Ok! I will switch off my fun mode for sometime"
                                emotion = Emotion.neutral
                                self.chatbot.globals['fun'] = False
                            else:
                                response = "I am depressed. Is there anything which I hurt you? I apologize for that"
                                emotion = Emotion.depressed
                    else:
                        if any_in(APPRECIATION, parsed):
                            response = random_response(GRATIFY)
                            emotion = Emotion.angel
                            confidence = 0.8
                        else:
                            # FIXME : Make it more smart
                            response = random_response(SIT_AND_SMILE)
                            emotion = Emotion.lol
                            if confidence > 0.8:
                                confidence -= 0.2
            else:
                if 'i' in parsed:
                    response = "Its ok,  {}.".format(
                        CONSOLATION[randint(0, len(CONSOLATION)-1)])
                    emotion = Emotion.positive
                else:
                    # well, I don't want to say ( I don't know )
                    # FIXME : Use a better algorithm to detect sentences
                    reversed = reverse(parsed)
                    response = 'Why do you think {}?'.format(
                        ' '.join(reversed))
                    emotion = Emotion.dead

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import logging

from chatterbot.logic import LogicAdapter
from currency_converter import CurrencyConverter
from sugaroid.sugaroid import SugaroidStatement

from sugaroid.brain.ooo import Emotion


class SugaroidCurrency:
    def __init__(self):
        self.currency_api = CurrencyConverter()

    def convert(self, src: str, dest: str, amount: float):
        if (src in self.currency_api.currencies) and (dest in self.currency_api.currencies):
            return self.currency_api.convert(amount, src, dest)
        else:
            if src not in self.currency_api.currencies:
                bad_cur = src
            else:
                bad_cur = dest
            return 'Hmm. Seems like {} is not a recognized currency.'.format(bad_cur)


class CurrencyAdapter(LogicAdapter):
    """
    Gives a random response, because Sugaroid tries not to say I don't know
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.currencies_src_ord = None
        self.currencies_dest = None
        self.currencies_src = None
        self.tokenized = None

    def can_process(self, statement):
        self.tokenized = self.chatbot.lp.tokenize(
            str(statement).replace('$', ' USD ').replace(
                '₹', ' INR ').replace('€', ' EUR ').replace('£', ' GBP ')
        )
        self.currencies_dest = []
        self.currencies_src = None
        if len(self.tokenized) >= 3:
            for i in range(len(self.tokenized) - 1):
                if self.tokenized[i].tag_ == 'TO':
                    dst = str(self.tokenized[i + 1].text).upper()
                    if len(dst) < 4:
                        self.currencies_dest.append(dst)
                    try:
                        if len(self.tokenized[i - 1].lower_) < 4:
                            self.currencies_src = str(
                                self.tokenized[i - 1].text).upper()
                    except IndexError:
                        pass
                elif self.tokenized[i].lower_ == 'is':

                    for j in range(i+1, len(self.tokenized)):
                        if self.tokenized[j].tag_ == 'IN':
                            dst = str(self.tokenized[j + 1].text).upper()
                            if len(dst) < 4:
                                self.currencies_dest.append(dst)
                            try:
                                src = str(self.tokenized[j-1].text).upper()
                                if len(src) < 4:
                                    self.currencies_src = src
                            except IndexError:
                                pass
                    if self.currencies_dest and self.currencies_src:
                        return True
                    else:
                        return False
                elif self.tokenized[i].tag_ == 'IN':
                    dst = str(self.tokenized[i + 1].text).upper()
                    if len(dst) < 4:
                        self.currencies_dest.append(dst)

        if self.currencies_dest and self.currencies_src:
            logging.info("CurrencyAdapter: Recognized source and destination currency types. src: {} and dest: {}"
                         .format(self.currencies_src, self.currencies_dest))
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        emotion = Emotion.rich
        confidence = 0.9
        response = None
        converted = []

        for i in self.tokenized:
            if i.tag_ in ['LS', 'CD']:
                self.currencies_src_ord = i.text

        if self.currencies_src_ord:
            try:
                self.currencies_src_ord = float(self.currencies_src_ord)
                sg_currency = SugaroidCurrency()
                for destination in self.currencies_dest:
                    converted.append('{} {}'.format(
                        sg_currency.convert(self.currencies_src.upper(
                        ), destination.upper(), self.currencies_src_ord),
                        destination.upper())
                    )
                response = ' '.join(converted)
            except ValueError:
                response = 'Seems like I cannot process {}. Maybe try a numerical value for me to understand better' \
                    .format(self.currencies_src_ord)

        else:
            response = 'Seems like you forgot the important part of your currency conversion statement. The number!'
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement
